/**
 * @file
 * test.adjustColorBrightness node implementation.
 *
 * @copyright Copyright © 2012–2014 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"

#include "VuoImageRenderer.h"

VuoModuleMetadata({
                     "title" : "Julia Filter",
                     "description" : "Julia fractal textured from input texture. Based on iq's 'Deform(2009)'. Code additions to add filter functions, 2011-2014, George Toledo.",
					 "keywords" : [ ],
					 "version" : "1.0.0",
					 "dependencies" : [
						 "VuoImageRenderer"
					 ],
					 "node": {
						 "isInterface" : false
					 }
				 });

#include "node.h"

static const char * fragmentShaderSource = VUOSHADER_GLSL_SOURCE(120,
	// Inputs

    uniform float time;
    uniform float zoom;
    uniform vec4 color;
    uniform float X;
    uniform float Y;
    uniform float brightness;
    uniform int iterations;

    uniform sampler2D texture;

	varying vec4 fragmentTextureCoordinate;

    //Deform by iq (2009)

    void main(void)
    {
        vec2 p = -1.0 + 2.0 * fragmentTextureCoordinate.xy;
        //vec2 p = fragmentTextureCoordinate.xy;

        vec2 cc = vec2( X, Y );

        float dmin = 1000.0;
        vec2 z  = p*vec2(1.33,1.0);
        //for( int i=0; i<64; i++ )
        for( int i=0; i<iterations; i++ )

        {
            z = cc + vec2( z.x*z.x - z.y*z.y, 2.0*z.x*z.y );
            float m2 = dot(z,z);
            if( m2>100.0 ) break;
            dmin=min(dmin,m2);
            }

        float clr = sqrt(sqrt(dmin))*(0.7*brightness);
       vec4 tx=texture2D(texture,z/(zoom*100.));

        gl_FragColor = color*vec4(vec3(clr),1.0)*tx;
    }

);
struct nodeInstanceData
{
	VuoShader shader;
	VuoGlContext glContext;
	VuoImageRenderer imageRenderer;
};

struct nodeInstanceData * nodeInstanceInit(void)
{
	struct nodeInstanceData * instance = (struct nodeInstanceData *)malloc(sizeof(struct nodeInstanceData));
	VuoRegister(instance, free);

	instance->glContext = VuoGlContext_use();

    instance->shader = VuoShader_make("Julia Filter");
	VuoShader_addSource(instance->shader, VuoMesh_IndividualTriangles, NULL, NULL, fragmentShaderSource);
	VuoRetain(instance->shader);

	instance->imageRenderer = VuoImageRenderer_make(instance->glContext);
	VuoRetain(instance->imageRenderer);

	return instance;
}

void nodeInstanceEvent
(
		VuoInstanceData(struct nodeInstanceData *) instance,
		VuoInputData(VuoImage) image,
        VuoInputData(VuoColor,{"default":{"r":1,"g":1,"b":1,"a":1}}) color,
        VuoInputData(VuoReal, {"default":0.0,"suggestedStep":0.1}) time,
        VuoInputData(VuoReal, {"default":1.0,"suggestedStep":1.}) zoomTexture,
        VuoInputData(VuoReal, {"default":-1.0,"suggestedStep":0.1}) X,
        VuoInputData(VuoReal, {"default":-0.15,"suggestedStep":0.1}) Y,
        VuoInputData(VuoReal, {"default":1.0,"suggestedStep":1.0}) gradBrightness,
        VuoInputData(VuoInteger, {"default":64,"suggestedStep":1}) Iterations,

		VuoOutputData(VuoImage) adjustedImage
)
{
	if (! image)
		return;
    VuoPoint4d col = { color.r, color.g, color.b, color.a };
	// Feed parameters to the shader.
	VuoShader_setUniform_VuoImage((*instance)->shader, "texture", image);
    VuoShader_setUniform_VuoReal ((*instance)->shader, "time",   time);
    VuoShader_setUniform_VuoReal ((*instance)->shader, "zoom",   zoomTexture);
    VuoShader_setUniform_VuoReal ((*instance)->shader, "X",   X);
    VuoShader_setUniform_VuoReal ((*instance)->shader, "Y",   Y);
    VuoShader_setUniform_VuoReal ((*instance)->shader, "brightness",   gradBrightness);
    VuoShader_setUniform_VuoInteger ((*instance)->shader, "iterations",   Iterations);

    VuoShader_setUniform_VuoPoint4d((*instance)->shader, "color", col);


    //VuoShader_setUniform_VuoInteger ((*instance)->shader, "iterations",   iterations);
    // VuoShader_setUniform_VuoReal ((*instance)->shader, "ResX",   image->pixelsWide);
    //VuoShader_setUniform_VuoReal ((*instance)->shader, "ResY",   image->pixelsHigh);


	// Render.
	*adjustedImage = VuoImageRenderer_draw((*instance)->imageRenderer, (*instance)->shader, image->pixelsWide, image->pixelsHigh, VuoImage_getColorDepth(image));
}

void nodeInstanceFini(VuoInstanceData(struct nodeInstanceData *) instance)
{
	VuoRelease((*instance)->shader);
	VuoRelease((*instance)->imageRenderer);
	VuoGlContext_disuse((*instance)->glContext);
}
